<?php

namespace Drp\NativeEventApp\Models;

use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\Token;

class Device extends Model
{
    use Notifiable;

    /** @var string */
    protected $table = 'devices';

    /** @var string */
    protected $keyType = 'string';

    /** @var bool */
    public $incrementing = false;

    protected $fillable = [
        'id',
        'auth_identifier',
        'access_token_id',
        'firebase_device_id',
    ];

    public function token(): BelongsTo
    {
        return $this->belongsTo(Token::class, 'access_token_id', 'id');
    }

    public function associateAccessToken(Token $token): self
    {
        return tap($this, static function (self $device) use ($token): void {
            $device->token()->associate($token);
            $device->save();
        });
    }

    public function revokeExistingAccessToken(): ?bool
    {
        return optional($this->token)->revoke();
    }

    public static function storeFirebaseDeviceId(Token $token, string $firebaseId): bool
    {
        return self::where('access_token_id', $token->id)
            ->update(['firebase_device_id' => $firebaseId]);
    }

    public static function removeFirebaseDeviceId(Token $token): bool
    {
        return self::where('access_token_id', $token->id)
            ->update(['firebase_device_id' => null]);
    }

    public static function getByToken(Token $token): ?self
    {
        return self::where('access_token_id', $token->id)->first();
    }

    public function routeNotificationForFcm(): ?string
    {
        return $this->firebase_device_id;
    }
}
