<?php

namespace Drp\NativeEventApp\Test\Api\Auth;

use Carbon\Carbon;
use Drp\NativeEventApp\Exceptions\FailedAuthentication;
use Drp\NativeEventApp\Http\Middleware\Authenticate;
use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Drp\NativeEventApp\Test\Support\InteractsWithAuth;
use Illuminate\Support\Facades\DB;
use Laravel\Passport\Passport;
use Laravel\Passport\Token;
use League\OAuth2\Server\Exception\OAuthServerException;
use Mockery;
use Spatie\Snapshots\MatchesSnapshots;

class AuthTest extends GraphQLTestCase
{
    use MatchesSnapshots, InteractsWithAuth;

    /** @test */
    public function queries_are_protected(): void
    {
        $query = '{
            events(id:1){
                id
            }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function revoked_tokens_cannot_be_used(): void
    {
        $token = $this->createAccessToken('accessToken');
        Passport::token()->first()->update(['revoked' => true]);

        $query = '{
            events(id:1){
                id
            }
        }';

        $headers = ['Authorization' => "Bearer $token"];

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query, $headers));
    }

    /** @test */
    public function deletes_access_token_and_refresh_token_on_logout(): void
    {
        $token = $this->createAccessToken('accessToken');

        $this->assertEquals(1, DB::table('oauth_access_tokens')->count());
        $this->assertEquals(1, DB::table('oauth_refresh_tokens')->count());
        $this->assertEquals(1, DB::table('oauth_auth_codes')->count());

        $query = 'mutation {
              logout {
                status
                message
              }
            }';

        $headers = ['Authorization' => "Bearer $token"];

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query, $headers));

        $this->assertEquals(0, DB::table('oauth_access_tokens')->count());
        $this->assertEquals(0, DB::table('oauth_refresh_tokens')->count());
    }
}
